#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Ejemplo de un ajuste lineal
"""

# Importamos librerías estándares

from numpy import *
import matplotlib.pyplot as plt

#%% fabrico datos aleatoreos **************************************************

random.seed(0)


xx = linspace(0,10,20)
yy = xx * 0.74 + 1.5
datos_y = yy +random.normal(0,0.5,20)
datos_x = xx+ +random.normal(0,0.05,20)

err_y = random.normal(0.5,0.1,20)

#%% Grafico los datos en crudo ************************************************

plt.figure(1)
plt.errorbar( datos_x, datos_y, xerr=0.05, yerr=err_y, 
            fmt='s', mfc='none', elinewidth = 1, capsize=3 , label='datos')

plt.grid(True,ls=':')
plt.xlabel('X')
plt.ylabel('Y')
plt.tight_layout()


#%% Acá hacemos el ajuste basico **********************************************

# Ajuste con una lineal
param = polyfit(datos_x,datos_y,1)


print('A*x+B')
print('A:',param[0] , '   B:',param[1])


modelo_x = linspace(-1,12)
modelo_y = polyval(param,modelo_x)


plt.figure(2)
plt.errorbar( datos_x, datos_y, xerr=0.05, yerr=err_y, 
            fmt='s', mfc='none', elinewidth = 1, capsize=3 , label='datos')
plt.plot(modelo_x, modelo_y  ,'-', label='modelo')

plt.grid(True,ls=':')
plt.xlabel('X')
plt.ylabel('Y')
plt.tight_layout()
# plt.savefig('02_07_ajuste_lineal.png')

chi2 = sum( (datos_y - polyval(param,datos_x))**2 )
r    = corrcoef(datos_x,datos_y)[0,1]

r2   = r**2


#%% Acá avanzado **************************************************************

parametros , pcov = polyfit(datos_x,datos_y,1 , cov=True)

# parametros: parametros ajustados.  Y = parametros[0] * X + parametros[1]
# pcov      : Matris de covarianza DE LOS PARÁMETROS

print('A*x+B')
print('A:',parametros[0] , '   B:',parametros[1])


# Calculamos la predicción que hace el modelo para los parámetros hallados
prediccion_modelo = polyval(parametros,datos_x)

chi2 = sum( residuos**2 )
# 142.3236051732523


# Calculamos el r-Pearson
r    = corrcoef(datos_y,prediccion_modelo)[0,1]  # Coeficiente de Pearson
# 0.9712166808776288

r2   = r**2
# 0.9432618412149578


# Calculamos alguna cantidades que nos sirven para algunos estimadores
N         = len(datos_y)   # Número de pares de datos ajustados
P         = 2         # Cantidad de parámetros del modelo. Para una lineal, son dos: a y b
SSE       = sum(( datos_y - prediccion_modelo )**2 )    # Resitual Sum of Squares
SST       = sum(( datos_y - mean(datos_y))**2)               # Total Sum of Squares

# http://en.wikipedia.org/wiki/Coefficient_of_determination
# Expresa el porcentaje de la varianza que logra explicar el modelos propuesto
R2        =  1 - SSE/SST                           # Coeficiente de determinación
R2_adj    = 1-(1-R2) * (N-1)/(N-P-1)              # Coeficiente de determinación Ajustado   

# El coeficiente R2 ajustado hace una corrección para evitar un sesgo que se produce
# Cuando tenemos un número de parámetros alto comparado con el número de datos.
# Es el que suele ser reportado en un informe de laboratorio

# El error estandar es la raiz de la varianza
# En la diagonal de la matris de covarianza de los parámetros está la varianza de
# cada parámetro en particular. Extrayendo la diagonal y calculando la raíz tenemos
# el Error Estandar
SE        = sqrt(diag( pcov  ))                        # Standar Error / Error estandar de los parámetros


# Cargamos función estadística t-student que nos permite calcular a partir de
# el error estandar cual es el intervalo de confianza de un parámetro
from scipy.stats.distributions import  t


# Para un intervalo de 95% de confianza, necesitamos un alpha de 5% == 0.05
alpha=0.05

sT = t.ppf(1.0 - alpha/2.0, N - P ) # student T multiplier
CI = sT * SE

print('R-squared    ',R2)
print('R-sq_adjusted',R2_adj)
#print('chi2_test    ',chi2_test)
print('r-pearson    ',r)
#print('p-value      ',p_val)
print('')
print('Error Estandard (SE):')
for i in range(P):
    print('parametro[{:3d}]: '.format(i) , parametros[i], ' ± ' , SE[i])
print('')
print('Intervalo de confianza al '+str((1-alpha)*100)+'%:')
for i in range(P):
    print('parametro[{:3d}]: '.format(i) , parametros[i], ' ± ' , CI[i])



#################################### grafico


modelo_x     = linspace(-1,12)
modelo_y     = polyval(parametros,modelo_x)
modelo_y_err = sqrt( pcov[0,0] * modelo_x**2  + pcov[1,1] + 2*modelo_x*pcov[0,1] )
residuos     = polyval(parametros,datos_x) - datos_y


fig, axx = plt.subplots( 2,1, figsize=(12,6) ,  constrained_layout=True, sharex=True, 
                        gridspec_kw=dict(height_ratios=[1,0.3]))
fig.set_constrained_layout_pads(w_pad=2/72, h_pad=2/72, hspace=0, wspace=0)


ax = axx[0]
ax.errorbar( datos_x, datos_y, xerr=0.05, yerr=err_y, 
            fmt='.',  capsize=3 , label='datos')
ax.plot(modelo_x, modelo_y  ,'-', label='modelo')

ax.fill_between( modelo_x , modelo_y-modelo_y_err , modelo_y+modelo_y_err , 
                alpha=0.2 , color='C3' , label='incerteza modelo')

ax.set_ylabel('Y')

ax = axx[1]
ax.plot(datos_x, residuos, '.')
ax.set_xlabel('X')
ax.set_ylabel('residuos')


for ax in axx:
    ax.grid(True,ls=':')

